@@ -20,7 +20,6 @@ import appdaemon.http as adhttp
 import appdaemon.logging as logging
 import appdaemon.utils as utils
 import pytz
-import yaml
 
 try:
     import pid
@@ -120,7 +119,6 @@ class ADMain:
         """
 
         try:
-
             # if to use uvloop
             if appdaemon.get("uvloop") is True and uvloop:
                 self.logger.info("Running AD using uvloop")
@@ -238,14 +236,23 @@ class ADMain:
         parser.add_argument("-m", "--moduledebug", nargs=2, action="append")
         parser.add_argument("-v", "--version", action="version", version="%(prog)s " + utils.__version__)
         parser.add_argument("--profiledash", help=argparse.SUPPRESS, action="store_true")
+        parser.add_argument("--toml", help="use TOML for configuration files", action="store_true")
 
         args = parser.parse_args()
 
         config_dir = args.config
         pidfile = args.pidfile
 
+        module_debug = {}
+        if args.moduledebug is not None:
+            for arg in args.moduledebug:
+                module_debug[arg[0]] = arg[1]
+
         if args.configfile is None:
-            config_file = "appdaemon.yaml"
+            if args.toml is True:
+                config_file = "appdaemon.toml"
+            else:
+                config_file = "appdaemon.yaml"
         else:
             config_file = args.configfile
 
@@ -259,81 +266,11 @@ class ADMain:
             parser.print_help()
             sys.exit(1)
 
-        module_debug = {}
-        if args.moduledebug is not None:
-            for arg in args.moduledebug:
-                module_debug[arg[0]] = arg[1]
-
-        #
-        # First locate secrets file
-        #
         try:
-
-            #
-            # Read config file using include directory
-            #
-
-            yaml.add_constructor("!include", utils._include_yaml, Loader=yaml.SafeLoader)
-
-            #
-            # Read config file using environment variables
-            #
-
-            yaml.add_constructor("!env_var", utils._env_var_yaml, Loader=yaml.SafeLoader)
-
-            #
-            # Initially load file to see if secret directive is present
-            #
-            yaml.add_constructor("!secret", utils._dummy_secret, Loader=yaml.SafeLoader)
-            with open(config_file_yaml, "r") as yamlfd:
-                config_file_contents = yamlfd.read()
-
-            config = yaml.load(config_file_contents, Loader=yaml.SafeLoader)
-
-            if "secrets" in config:
-                secrets_file = config["secrets"]
-            else:
-                secrets_file = os.path.join(os.path.dirname(config_file_yaml), "secrets.yaml")
-
-            #
-            # Read Secrets
-            #
-            if os.path.isfile(secrets_file):
-                with open(secrets_file, "r") as yamlfd:
-                    secrets_file_contents = yamlfd.read()
-
-                utils.secrets = yaml.load(secrets_file_contents, Loader=yaml.SafeLoader)
-
-            else:
-                if "secrets" in config:
-                    print(
-                        "ERROR",
-                        "Error loading secrets file: {}".format(config["secrets"]),
-                    )
-                    sys.exit()
-
-            #
-            # Read config file again, this time with secrets
-            #
-
-            yaml.add_constructor("!secret", utils._secret_yaml, Loader=yaml.SafeLoader)
-
-            with open(config_file_yaml, "r") as yamlfd:
-                config_file_contents = yamlfd.read()
-
-            config = yaml.load(config_file_contents, Loader=yaml.SafeLoader)
-
-        except yaml.YAMLError as exc:
-            print("ERROR", "Error loading configuration")
-            if hasattr(exc, "problem_mark"):
-                if exc.context is not None:
-                    print("ERROR", "parser says")
-                    print("ERROR", str(exc.problem_mark))
-                    print("ERROR", str(exc.problem) + " " + str(exc.context))
-                else:
-                    print("ERROR", "parser says")
-                    print("ERROR", str(exc.problem_mark))
-                    print("ERROR", str(exc.problem))
+            config = utils.read_config_file(config_file_yaml)
+        except Exception as e:
+            print(f"Unexpected error loading config file: {config_file_yaml}")
+            print(e)
             sys.exit()
 
         if "appdaemon" not in config:
@@ -344,6 +281,7 @@ class ADMain:
         if "disable_apps" not in appdaemon:
             appdaemon["disable_apps"] = False
 
+        appdaemon["use_toml"] = args.toml
         appdaemon["config_dir"] = config_dir
         appdaemon["config_file"] = config_file_yaml
         appdaemon["app_config_file"] = os.path.join(os.path.dirname(config_file_yaml), "apps.yaml")
