@@ -12,9 +12,21 @@ class NamespaceException(Exception):
     pass
 
 
+class DomainException(Exception):
+    pass
+
+
+class ServiceException(Exception):
+    pass
+
+
 class AppException(Exception):
     pass
 
 
 class HandlerException(Exception):
     pass
+
+
+class TimeOutException(Exception):
+    pass
