@@ -28,7 +28,6 @@ def securedata(myfunc):
     """
 
     async def wrapper(*args):
-
         self = args[0]
         request = args[1]
         if self.password is None:
@@ -58,7 +57,6 @@ def secure(myfunc):
     """
 
     async def wrapper(*args):
-
         self = args[0]
         request = args[1]
         if self.password is None:
@@ -87,7 +85,6 @@ def route_secure(myfunc):
     """
 
     async def wrapper(*args):
-
         self = args[0]
         request = args[1]
         if self.password is None or self.valid_tokens == []:
@@ -111,7 +108,6 @@ def route_secure(myfunc):
 
 class HTTP:
     def __init__(self, ad: AppDaemon, loop, logging, appdaemon, dashboard, old_admin, admin, api, http):
-
         self.AD = ad
         self.logging = logging
         self.logger = ad.logging.get_child("_http")
@@ -228,7 +224,6 @@ class HTTP:
                 admin_args = old_admin
 
             if old_admin is not None or admin is not None:
-
                 self.admin_obj = adadmin.Admin(
                     self.config_dir,
                     logging,
@@ -368,7 +363,6 @@ class HTTP:
         self._process_arg("static_dirs", http)
 
     async def start_server(self):
-
         self.logger.info("Running on port %s", self.port)
 
         self.runner = web.AppRunner(self.app)
@@ -542,7 +536,6 @@ class HTTP:
 
     @securedata
     async def get_namespace_entities(self, request):
-
         namespace = None
         try:
             namespace = request.match_info.get("namespace")
@@ -567,7 +560,6 @@ class HTTP:
 
     @securedata
     async def get_namespaces(self, request):
-
         try:
             self.logger.debug("get_namespaces() called)")
             state = await self.AD.state.list_namespaces()
@@ -584,7 +576,6 @@ class HTTP:
 
     @securedata
     async def get_services(self, request):
-
         try:
             self.logger.debug("get_services() called)")
             state = self.AD.services.list_services()
@@ -746,6 +737,7 @@ class HTTP:
         self.app.router.add_get("/api/appdaemon/state", self.get_state)
         self.app.router.add_get("/api/appdaemon/logs", self.get_logs)
         self.app.router.add_post("/api/appdaemon/{endpoint}", self.call_app_endpoint)
+        self.app.router.add_get("/api/appdaemon/{endpoint}", self.call_app_endpoint)
         self.app.router.add_get("/api/appdaemon", self.get_ad)
 
     def setup_http_routes(self):
@@ -869,8 +861,7 @@ class HTTP:
         return web.Response(text=res, content_type="text/html")
 
     @securedata
-    async def call_app_endpoint(self, request):
-
+    async def call_app_endpoint(self, request):  # @next-release get requests object in somehow
         code = 200
         ret = ""
         endpoint = request.match_info.get("endpoint")
@@ -899,7 +890,6 @@ class HTTP:
     # Routes, Status and Templates
 
     async def register_endpoint(self, cb: Callable, endpoint: str, name: str, **kwargs: Optional[dict]) -> str:
-
         handle = uuid.uuid4().hex
 
         # first we check to ensure that endpoint not been used before
@@ -923,6 +913,7 @@ class HTTP:
     async def dispatch_app_endpoint(self, endpoint, request):
         callback = None
         rargs = {}
+        appname = None
 
         for name in self.app_endpoints:
             if callback is not None:  # a callback has been collected
@@ -934,18 +925,33 @@ class HTTP:
                 if app_endpoint == endpoint:
                     callback = self.app_endpoints[name][handle]["callback"]
                     rargs.update(self.app_endpoints[name][handle]["kwargs"])
+                    appname = name
                     break
 
         if callback is not None:
-            if asyncio.iscoroutinefunction(callback):
-                return await callback(request, rargs)
+            app_args = self.AD.app_management.app_config[appname]
+            if "use_dictionary_unpacking" in app_args:
+                use_dictionary_unpacking = app_args["use_dictionary_unpacking"]
             else:
+                use_dictionary_unpacking = self.AD.use_dictionary_unpacking
+            if request.method == "POST":
                 try:
                     args = await request.json()
                 except json.decoder.JSONDecodeError:
                     return self.get_response(request, 400, "JSON Decode Error")
+            else:
+                args = request.query
 
-                return await utils.run_in_executor(self, callback, args, rargs)
+            if asyncio.iscoroutinefunction(callback):
+                if use_dictionary_unpacking is True:
+                    return await callback(args, **rargs)
+                else:
+                    return await callback(args, rargs)
+            else:
+                if use_dictionary_unpacking is True:
+                    return await utils.run_in_executor(self, callback, args, **rargs)
+                else:
+                    return await utils.run_in_executor(self, callback, args, rargs)
         else:
             return "", 404
 
@@ -953,7 +959,6 @@ class HTTP:
     # App based Web Server
     #
     async def register_route(self, cb: Callable, route: str, name: str, **kwargs: Optional[dict]) -> str:
-
         if not asyncio.iscoroutinefunction(cb):  # must be async function
             self.logger.warning(
                 "Could not Register Callback for %s, using Route %s as Web Server Route. Callback must be Async",
@@ -984,7 +989,6 @@ class HTTP:
 
     @route_secure
     async def app_webserver(self, request):
-
         name = None
         route = request.match_info.get("route")
 
