@@ -16,7 +16,6 @@ class PluginBase:
     """
 
     def __init__(self, ad: AppDaemon, name, args):
-
         self.AD = ad
         self.logger = self.AD.logging.get_child(name)
 
@@ -25,11 +24,9 @@ class PluginBase:
 
 
 class Plugins:
-
     required_meta = ["latitude", "longitude", "elevation", "time_zone"]
 
     def __init__(self, ad: AppDaemon, kwargs):
-
         self.AD = ad
         self.plugins = kwargs
         self.stopping = False
@@ -38,6 +35,8 @@ class Plugins:
         self.plugin_objs = {}
         self.last_plugin_state = {}
 
+        self.perf_count = 0
+
         self.logger = ad.logging.get_child("_plugin_management")
         self.error = self.AD.logging.get_error()
 
@@ -69,7 +68,6 @@ class Plugins:
                 if "disable" in self.plugins[name] and self.plugins[name]["disable"] is True:
                     self.logger.info("Plugin '%s' disabled", name)
                 else:
-
                     if "refresh_delay" not in self.plugins[name]:
                         self.plugins[name]["refresh_delay"] = 600
 
@@ -105,7 +103,6 @@ class Plugins:
                             module_name,
                         )
                     try:
-
                         mod = __import__(full_module_name, globals(), locals(), [module_name], 0)
 
                         app_class = getattr(mod, class_name)
@@ -145,11 +142,32 @@ class Plugins:
 
     def run_plugin_utility(self):
         for plugin in self.plugin_objs:
-            if hasattr(self.plugin_objs[plugin]["object"].utility(), "utility"):
+            if hasattr(self.plugin_objs[plugin]["object"], "utility"):
                 self.plugin_objs[plugin]["object"].utility()
 
-    def process_meta(self, meta, namespace):
+    async def get_plugin_perf_data(self):
+        # Grab stats every 10th time we are called (this will be roughly a 10 second average)
+        self.perf_count += 1
 
+        if self.perf_count < 10:
+            return
+
+        self.perf_count = 0
+
+        for plugin in self.plugin_objs:
+            if hasattr(self.plugin_objs[plugin]["object"], "perf_data"):
+                p_data = await self.plugin_objs[plugin]["object"].perf_data()
+                await self.AD.state.set_state(
+                    "plugin",
+                    "admin",
+                    f"plugin.{self.get_plugin_from_namespace(plugin)}",
+                    bytes_sent_ps=round(p_data["bytes_sent"] / p_data["duration"], 1),
+                    bytes_recv_ps=round(p_data["bytes_recv"] / p_data["duration"], 1),
+                    requests_sent_ps=round(p_data["requests_sent"] / p_data["duration"], 1),
+                    updates_recv_ps=round(p_data["updates_recv"] / p_data["duration"], 1),
+                )
+
+    def process_meta(self, meta, namespace):
         if meta is not None:
             for key in self.required_meta:
                 if getattr(self.AD, key) is None:
@@ -206,7 +224,6 @@ class Plugins:
 
                 if namespaces != []:  # there are multiple namesapces
                     for namesp in namespaces:
-
                         if state[namesp] is not None:
                             await utils.run_in_executor(
                                 self,
@@ -246,7 +263,10 @@ class Plugins:
                     # Create plugin entity
                     #
                     await self.AD.state.add_entity(
-                        "admin", "plugin.{}".format(name), "active", {"totalcallbacks": 0, "instancecallbacks": 0}
+                        "admin",
+                        "plugin.{}".format(name),
+                        "active",
+                        {"bytes_sent_ps": 0, "bytes_recv_ps": 0, "requests_sent_ps": 0, "updates_recv_ps": 0},
                     )
 
                     self.logger.info("Got initial state from namespace %s", namespace)
