@@ -56,7 +56,9 @@ class HassPlugin(PluginBase):
         self.cert_verify = args.get("cert_verify")
         self.commtype = args.get("commtype", "WS")
         self.ha_key = args.get("ha_key")
-        self.ha_url = args.get("ha_url", "")
+
+        # Remove trailing slash if present
+        self.ha_url = args.get("ha_url", "").rstrip("/")
         self.namespace = args.get("namespace", "default")
         self.plugin_startup_conditions = args.get("plugin_startup_conditions", {})
         self.retry_secs = int(args.get("retry_secs", 5))
@@ -78,8 +80,39 @@ class HassPlugin(PluginBase):
         self.reading_messages = False
         self.stopping = False
 
+        # Performance Data
+
+        self.bytes_sent = 0
+        self.bytes_recv = 0
+        self.requests_sent = 0
+        self.updates_recv = 0
+        self.last_check_ts = 0
+
         self.logger.info("HASS Plugin initialization complete")
 
+    async def perf_data(self):
+        data = {
+            "bytes_sent": self.bytes_sent,
+            "bytes_recv": self.bytes_recv,
+            "requests_sent": self.requests_sent,
+            "updates_recv": self.updates_recv,
+            "duration": await self.AD.sched.get_now_ts() - self.last_check_ts,
+        }
+
+        self.bytes_sent = 0
+        self.bytes_recv = 0
+        self.requests_sent = 0
+        self.updates_recv = 0
+        self.last_check_ts = await self.AD.sched.get_now_ts()
+
+        return data
+
+    def update_perf(self, **kwargs):
+        self.bytes_sent += kwargs.get("bytes_sent", 0)
+        self.bytes_recv += kwargs.get("bytes_recv", 0)
+        self.requests_sent += kwargs.get("requests_sent", 0)
+        self.updates_recv += kwargs.get("updates_recv", 0)
+
     async def am_reading_messages(self):
         return self.reading_messages
 
@@ -167,7 +200,6 @@ class HassPlugin(PluginBase):
     # Get initial state
     #
     async def get_complete_state(self):
-
         hass_state = await self.get_hass_state()
         states = {}
         for state in hass_state:
@@ -186,7 +218,6 @@ class HassPlugin(PluginBase):
     # Handle state updates
     #
     async def evaluate_started(self, first_time, plugin_booting, event=None):  # noqa: C901
-
         if first_time is True:
             self.hass_ready = False
             self.state_matched = False
@@ -292,7 +323,6 @@ class HassPlugin(PluginBase):
     #
 
     async def get_updates(self):  # noqa: C901
-
         _id = 0
         self.already_notified = False
         self.first_time = True
@@ -379,6 +409,8 @@ class HassPlugin(PluginBase):
                     ret = await utils.run_in_executor(self, self.ws.recv)
                     result = json.loads(ret)
 
+                    self.update_perf(bytes_recv=len(ret), updates_recv=1)
+
                     if not (result["id"] == _id and result["type"] == "event"):
                         self.logger.warning("Unexpected result from Home Assistant, id = %s", _id)
                         self.logger.warning(result)
@@ -475,6 +507,7 @@ class HassPlugin(PluginBase):
                 txt = await r.text()
                 self.logger.warning("Code: %s, error: %s", r.status, txt)
                 state = None
+                self.update_perf(bytes_sent=len(json.dumps(kwargs)), bytes_recv=len(await r.text()), requests_sent=1)
             return state
         except (asyncio.TimeoutError, asyncio.CancelledError):
             self.logger.warning("Timeout in set_state(%s, %s, %s)", namespace, entity_id, kwargs)
@@ -518,7 +551,6 @@ class HassPlugin(PluginBase):
             api_url = "/api/services/{}/{}".format(domain, service)
 
         try:
-
             r = await self.session.post(api_url, json=data)
 
             if r.status == 200 or r.status == 201:
@@ -537,6 +569,7 @@ class HassPlugin(PluginBase):
                 self.logger.warning("Code: %s, error: %s", r.status, txt)
                 result = None
 
+            self.update_perf(bytes_sent=len(json.dumps(data)), bytes_recv=len(await r.text()), requests_sent=1)
             return result
         except (asyncio.TimeoutError, asyncio.CancelledError):
             self.logger.warning(
@@ -566,6 +599,9 @@ class HassPlugin(PluginBase):
             r = await self.session.get(api_url)
 
             if r.status == 200 or r.status == 201:
+                self.bytes_recv += len(await r.text())
+                self.updates_recv += 1
+
                 result = await r.json()
             else:
                 self.logger.warning("Error calling Home Assistant to get_history")
@@ -573,6 +609,7 @@ class HassPlugin(PluginBase):
                 self.logger.warning("Code: %s, error: %s", r.status, txt)
                 result = None
 
+            self.update_perf(bytes_sent=len(json.dumps(api_url)), bytes_recv=len(await r.text()), requests_sent=1)
             return result
 
         except aiohttp.client_exceptions.ServerDisconnectedError:
@@ -602,7 +639,7 @@ class HassPlugin(PluginBase):
             )
 
         if "end_time" in kwargkeys and {"start_time", "days"}.isdisjoint(kwargkeys):
-            raise ValueError(f"Can not have end_time without start_time or days")
+            raise ValueError("Can not have end_time without start_time or days")
 
         entity_id = kwargs.get("entity_id", "").strip()
         days = max(0, kwargs.get("days", 0))
@@ -646,7 +683,6 @@ class HassPlugin(PluginBase):
         return apiurl
 
     async def get_hass_state(self, entity_id=None):
-
         if entity_id is None:
             api_url = "/api/states"
         else:
@@ -660,6 +696,7 @@ class HassPlugin(PluginBase):
             txt = await r.text()
             self.logger.warning("Code: %s, error: %s", r.status, txt)
             state = None
+        self.update_perf(bytes_sent=len(json.dumps(api_url)), bytes_recv=len(await r.text()), requests_sent=1)
         return state
 
     def validate_meta(self, meta, key):
@@ -707,6 +744,7 @@ class HassPlugin(PluginBase):
             self.validate_meta(meta, "elevation")
             self.validate_tz(meta)
 
+            self.update_perf(bytes_sent=len(json.dumps(api_url)), bytes_recv=len(await r.text()), requests_sent=1)
             return meta
         except Exception as ex:
             self.logger.warning("Error getting metadata - retrying: %s", str(ex))
@@ -719,9 +757,12 @@ class HassPlugin(PluginBase):
             api_url = "/api/services"
             self.logger.debug("get_hass_services: url is %s", api_url)
             r = await self.session.get(api_url)
+
             r.raise_for_status()
             services = await r.json()
 
+            self.update_perf(bytes_sent=len(json.dumps(api_url)), bytes_recv=len(await r.text()), requests_sent=1)
+
             # manually added HASS services
             new_services = {}
             new_services["database"] = {"history": {}}
@@ -828,7 +869,11 @@ class HassPlugin(PluginBase):
         try:
             r = await self.session.post(api_url, json=kwargs)
             r.raise_for_status()
+
             state = await r.json()
+
+            self.update_perf(bytes_sent=len(json.dumps(kwargs)), bytes_recv=len(await r.text()), requests_sent=1)
+
             return state
         except (asyncio.TimeoutError, asyncio.CancelledError):
             self.logger.warning("Timeout in fire_event(%s, %s, %s)", event, namespace, kwargs)
@@ -854,6 +899,8 @@ class HassPlugin(PluginBase):
         try:
             r = await self.session.delete(api_url)
             if r.status == 200 or r.status == 201:
+                self.bytes_recv += len(await r.text())
+                self.updates_recv += 1
                 state = await r.json()
                 self.logger.debug("return = %s", state)
             else:
@@ -861,6 +908,8 @@ class HassPlugin(PluginBase):
                 txt = await r.text()
                 self.logger.warning("Code: %s, error: %s", r.status, txt)
                 state = None
+
+            self.update_perf(bytes_sent=len(json.dumps(api_url)), bytes_recv=len(await r.text()), requests_sent=1)
             return state
         except (asyncio.TimeoutError, asyncio.CancelledError):
             self.logger.warning("Timeout in remove_entity(%s, %s)", namespace, entity_id)
