@@ -93,7 +93,6 @@ class Scheduler:
         self.stopping = True
 
     async def insert_schedule(self, name, aware_dt, callback, repeat, type_, **kwargs):
-
         # aware_dt will include a timezone of some sort - convert to utc timezone
         utc = aware_dt.astimezone(pytz.utc)
 
@@ -389,11 +388,12 @@ class Scheduler:
         else:
             return self.next_sunset(day_offset)
 
-    def todays_sunrise(self, days_offset):
-        candidate_date = (self.now + datetime.timedelta(days=days_offset)).astimezone(self.AD.tz).date()
-        next_rising_dt = self.location.sunrise(date=candidate_date, local=False, observer_elevation=self.AD.elevation)
+    def todays_sunrise(self, days_offset):  # @here todays sunrise
+        candidate_date = (self.now.astimezone(self.AD.tz) + datetime.timedelta(days=days_offset)).date()
+        # self.logger.info(f"{self.now.astimezone(self.AD.tz)=}, {candidate_date=}")
+        sunrise = self.location.sunrise(date=candidate_date, local=True, observer_elevation=self.AD.elevation)
 
-        return next_rising_dt
+        return sunrise
 
     def next_sunrise(self, offset: int = 0):
         day_offset = 0
@@ -428,10 +428,11 @@ class Scheduler:
         return next_setting_dt
 
     def todays_sunset(self, days_offset):
-        candidate_date = (self.now + datetime.timedelta(days=days_offset)).astimezone(self.AD.tz).date()
-        next_setting_dt = self.location.sunset(date=candidate_date, local=False, observer_elevation=self.AD.elevation)
+        candidate_date = (self.now.astimezone(self.AD.tz) + datetime.timedelta(days=days_offset)).date()
+        # self.logger.info(f"{self.now.astimezone(self.AD.tz)=}, {candidate_date=}")
+        sunset = self.location.sunset(date=candidate_date, local=True, observer_elevation=self.AD.elevation)
 
-        return next_setting_dt
+        return sunset
 
     @staticmethod
     def get_offset(kwargs: dict):
@@ -464,7 +465,6 @@ class Scheduler:
     #
 
     def get_next_entries(self):
-
         next_exec = datetime.datetime.now(pytz.utc).replace(year=datetime.MAXYEAR, month=12, day=31)
         for name in self.schedule.keys():
             for entry in self.schedule[name].keys():
@@ -482,25 +482,6 @@ class Scheduler:
 
         return next_entries
 
-    async def process_dst(self, old, new):
-        #
-        # Rewrite timestamps to new local time
-        #
-        offset = old - new
-        self.logger.debug("Process_dst()")
-        self.logger.debug("offset  %s", offset)
-        for app in self.schedule:
-            for entry in self.schedule[app]:
-                args = self.schedule[app][entry]
-                # Sunrise and sunset will already be correct. Anything else needs to be reset to a new local time
-                self.logger.debug("Before rewrite: %s", args)
-                if args["type"] != "next_rising" and args["type"] != "next_setting":
-                    # If our interval is less than the jump don't rewrite the timestamp
-                    if float(args["interval"]) > abs(offset.total_seconds()):
-                        args["timestamp"] += offset
-                        args["basetime"] += offset
-                self.logger.debug("After rewrite: %s", args)
-
     def get_next_dst_offset(self, base, limit):
         #
         # I can't believe there isn't a better way to find the next DST transition but ...
@@ -509,8 +490,8 @@ class Scheduler:
         # true for every timezone. With this method, as long as pytz's dst() function is correct, this should work
         #
 
-        # TODO: Convert this to some sort of binary search for efficiency
-        # TODO: This really should support sub 1 second periods better
+        # @todo : Convert this to some sort of binary search for efficiency
+        # @todo : This really should support sub 1 second periods better
         self.logger.debug("get_next_dst_offset() base=%s limit=%s", base, limit)
         current = base.astimezone(self.AD.tz).dst()
         self.logger.debug("current=%s", current)
@@ -585,10 +566,8 @@ class Scheduler:
                 )
                 if old_dst_offset != dst_offset:
                     #
-                    # DST began or ended, we need to go fix any existing scheduler entries to match the new local time
-                    #
-                    self.logger.info("Daylight Savings Time transition detected - rewriting events to new local time")
-                    await self.process_dst(old_dst_offset, dst_offset)
+                    # DST began or ended, lets prove we noticed
+                    self.logger.info("Daylight Savings Time transition detected")
                     #
                     # Re calculate next entries
                     #
@@ -795,25 +774,31 @@ class Scheduler:
         else:
             now = (await self.get_now()).astimezone(self.AD.tz)
 
-        # self.diag.info(f"locals: {locals()}")
+        # self.logger.info(
+        #    "\n" + "-" * 80 + f"\nInitial\nstart = {start_time}\nnow   = {now}\nend   = {end_time}\n" + "-" * 80
+        # )
+
         # Comparisons
         if end_time < start_time:
-            # self.diag.info("Midnight transition")
             # Start and end time backwards.
             # Spans midnight
             # Lets start by assuming end_time is wrong and should be tomorrow
             # This will be true if we are currently after start_time
             end_time = (await self._parse_time(end_time_str, name, today=True, days_offset=1))["datetime"]
+            # self.logger.info(
+            #    f"\nMidnight transition detected\nstart = {start_time}\nnow   = {now}\nend   = {end_time}\n" + "-" * 80
+            # )
             if now < start_time and now < end_time:
-                # self.diag.info("Reverse")
                 # Well, it's complicated -
                 # We crossed into a new day and things changed.
                 # Now all times have shifted relative to the new day, so we need to look at it differently
                 # If both times are now in the future, we now actually want to set start time back a day and keep end_time as today
                 start_time = (await self._parse_time(start_time_str, name, today=True, days_offset=-1))["datetime"]
                 end_time = (await self._parse_time(end_time_str, name, today=True, days_offset=0))["datetime"]
+                # self.logger.info(f"\nReverse\nstart = {start_time}\nnow   = {now}\nend   = {end_time}\n" + "=" * 80)
 
-        # self.diag.info(f"\nstart = {start_time}\nnow   = {now}\nend   = {end_time}")
+        # self.logger.info(f"\nFinal\nstart = {start_time}\nnow   = {now}\nend   = {end_time}\n" + "-" * 80)
+        # self.logger.info(f"Final decision: {start_time <= now <= end_time}\n" + "=" * 80)
         return start_time <= now <= end_time
 
     async def sunset(self, aware, today=False, days_offset=0):
@@ -845,7 +830,7 @@ class Scheduler:
             return (
                 (await self._parse_time(time_str, name, today=today, days_offset=days_offset))["datetime"]
                 .astimezone(self.AD.tz)
-                .time()
+                .timetz()
             )
         else:
             return self.make_naive(
