@@ -17,7 +17,6 @@ from appdaemon.appdaemon import AppDaemon
 
 class Threading:
     def __init__(self, ad: AppDaemon, kwargs):
-
         self.AD = ad
         self.kwargs = kwargs
 
@@ -92,7 +91,6 @@ class Threading:
         self.current_callbacks_fired = 0
 
     async def init_admin_stats(self):
-
         # Initialize admin stats
 
         await self.add_entity("admin", "sensor.callbacks_total_fired", 0)
@@ -332,7 +330,6 @@ class Threading:
             if (
                 warning_step == 0 and warning_iterations >= self.AD.qsize_warning_iterations
             ) or warning_iterations == self.AD.qsize_warning_iterations:
-
                 for thread in self.threads:
                     qsize = self.threads[thread]["queue"].qsize()
                     if qsize > 0:
@@ -502,7 +499,6 @@ class Threading:
         self.threads[name]["thread"] = t
 
     async def calculate_pin_threads(self):
-
         if self.pin_threads == 0:
             return
 
@@ -761,7 +757,7 @@ class Threading:
                             __attribute=attribute,
                             __old_state=old,
                             __new_state=new,
-                            **kwargs
+                            **kwargs,
                         )
                     else:
                         #
@@ -874,13 +870,22 @@ class Threading:
         if "__silent" in args["kwargs"]:
             silent = args["kwargs"]["__silent"]
 
+        app_args = self.AD.app_management.app_config[args["name"]]
+        if "use_dictionary_unpacking" in app_args:
+            use_dictionary_unpacking = app_args["use_dictionary_unpacking"]
+        else:
+            use_dictionary_unpacking = self.AD.use_dictionary_unpacking
+
         app = await self.AD.app_management.get_app_instance(name, objectid)
         if app is not None:
             try:
                 if _type == "scheduler":
                     try:
                         await self.update_thread_info("async", callback, name, _type, _id, silent)
-                        await funcref(self.AD.sched.sanitize_timer_kwargs(app, args["kwargs"]))
+                        if use_dictionary_unpacking is True:
+                            await funcref(**self.AD.sched.sanitize_timer_kwargs(app, args["kwargs"]))
+                        else:
+                            await funcref(self.AD.sched.sanitize_timer_kwargs(app, args["kwargs"]))
                     except TypeError:
                         self.report_callback_sig(name, "scheduler", funcref, args)
 
@@ -891,13 +896,22 @@ class Threading:
                         old_state = args["old_state"]
                         new_state = args["new_state"]
                         await self.update_thread_info("async", callback, name, _type, _id, silent)
-                        await funcref(
-                            entity,
-                            attr,
-                            old_state,
-                            new_state,
-                            self.AD.state.sanitize_state_kwargs(app, args["kwargs"]),
-                        )
+                        if use_dictionary_unpacking is True:
+                            await funcref(
+                                entity,
+                                attr,
+                                old_state,
+                                new_state,
+                                **self.AD.state.sanitize_state_kwargs(app, args["kwargs"]),
+                            )
+                        else:
+                            await funcref(
+                                entity,
+                                attr,
+                                old_state,
+                                new_state,
+                                self.AD.state.sanitize_state_kwargs(app, args["kwargs"]),
+                            )
                     except TypeError:
                         self.report_callback_sig(name, "state", funcref, args)
 
@@ -905,14 +919,24 @@ class Threading:
                     data = args["data"]
                     try:
                         await self.update_thread_info("async", callback, name, _type, _id, silent)
-                        await funcref(
-                            data["app_name"],
-                            data["ts"],
-                            data["level"],
-                            data["log_type"],
-                            data["message"],
-                            self.AD.events.sanitize_event_kwargs(app, args["kwargs"]),
-                        )
+                        if use_dictionary_unpacking is True:
+                            await funcref(
+                                data["app_name"],
+                                data["ts"],
+                                data["level"],
+                                data["log_type"],
+                                data["message"],
+                                **self.AD.logging.sanitize_log_kwargs(app, args["kwargs"]),
+                            )
+                        else:
+                            await funcref(
+                                data["app_name"],
+                                data["ts"],
+                                data["level"],
+                                data["log_type"],
+                                data["message"],
+                                self.AD.logging.sanitize_log_kwargs(app, args["kwargs"]),
+                            )
                     except TypeError:
                         self.report_callback_sig(name, "log_event", funcref, args)
 
@@ -920,7 +944,14 @@ class Threading:
                     data = args["data"]
                     try:
                         await self.update_thread_info("async", callback, name, _type, _id, silent)
-                        await funcref(args["event"], data, self.AD.events.sanitize_event_kwargs(app, args["kwargs"]))
+                        if use_dictionary_unpacking is True:
+                            await funcref(
+                                args["event"], data, **self.AD.events.sanitize_event_kwargs(app, args["kwargs"])
+                            )
+                        else:
+                            await funcref(
+                                args["event"], data, self.AD.events.sanitize_event_kwargs(app, args["kwargs"])
+                            )
                     except TypeError:
                         self.report_callback_sig(name, "event", funcref, args)
 
@@ -962,6 +993,12 @@ class Threading:
             if "__silent" in args["kwargs"]:
                 silent = args["kwargs"]["__silent"]
 
+            app_args = self.AD.app_management.app_config[args["name"]]
+            if "use_dictionary_unpacking" in app_args:
+                use_dictionary_unpacking = app_args["use_dictionary_unpacking"]
+            else:
+                use_dictionary_unpacking = self.AD.use_dictionary_unpacking
+
             app = utils.run_coroutine_threadsafe(self, self.AD.app_management.get_app_instance(name, objectid))
             if app is not None:
                 try:
@@ -971,7 +1008,10 @@ class Threading:
                                 self,
                                 self.update_thread_info(thread_id, callback, name, _type, _id, silent),
                             )
-                            funcref(self.AD.sched.sanitize_timer_kwargs(app, args["kwargs"]))
+                            if use_dictionary_unpacking is True:
+                                funcref(**self.AD.sched.sanitize_timer_kwargs(app, args["kwargs"]))
+                            else:
+                                funcref(self.AD.sched.sanitize_timer_kwargs(app, args["kwargs"]))
                         except TypeError:
                             self.report_callback_sig(name, "scheduler", funcref, args)
 
@@ -985,13 +1025,22 @@ class Threading:
                                 self,
                                 self.update_thread_info(thread_id, callback, name, _type, _id, silent),
                             )
-                            funcref(
-                                entity,
-                                attr,
-                                old_state,
-                                new_state,
-                                self.AD.state.sanitize_state_kwargs(app, args["kwargs"]),
-                            )
+                            if use_dictionary_unpacking is True:
+                                funcref(
+                                    entity,
+                                    attr,
+                                    old_state,
+                                    new_state,
+                                    **self.AD.state.sanitize_state_kwargs(app, args["kwargs"]),
+                                )
+                            else:
+                                funcref(
+                                    entity,
+                                    attr,
+                                    old_state,
+                                    new_state,
+                                    self.AD.state.sanitize_state_kwargs(app, args["kwargs"]),
+                                )
                         except TypeError:
                             self.report_callback_sig(name, "state", funcref, args)
 
@@ -1002,14 +1051,24 @@ class Threading:
                                 self,
                                 self.update_thread_info(thread_id, callback, name, _type, _id, silent),
                             )
-                            funcref(
-                                data["app_name"],
-                                data["ts"],
-                                data["level"],
-                                data["log_type"],
-                                data["message"],
-                                self.AD.events.sanitize_event_kwargs(app, args["kwargs"]),
-                            )
+                            if use_dictionary_unpacking is True:
+                                funcref(
+                                    data["app_name"],
+                                    data["ts"],
+                                    data["level"],
+                                    data["log_type"],
+                                    data["message"],
+                                    **self.AD.logging.sanitize_log_kwargs(app, args["kwargs"]),
+                                )
+                            else:
+                                funcref(
+                                    data["app_name"],
+                                    data["ts"],
+                                    data["level"],
+                                    data["log_type"],
+                                    data["message"],
+                                    self.AD.logging.sanitize_log_kwargs(app, args["kwargs"]),
+                                )
                         except TypeError:
                             self.report_callback_sig(name, "log_event", funcref, args)
 
@@ -1020,7 +1079,12 @@ class Threading:
                                 self,
                                 self.update_thread_info(thread_id, callback, name, _type, _id, silent),
                             )
-                            funcref(args["event"], data, self.AD.events.sanitize_event_kwargs(app, args["kwargs"]))
+                            if use_dictionary_unpacking is True:
+                                funcref(
+                                    args["event"], data, **self.AD.events.sanitize_event_kwargs(app, args["kwargs"])
+                                )
+                            else:
+                                funcref(args["event"], data, self.AD.events.sanitize_event_kwargs(app, args["kwargs"]))
                         except TypeError:
                             self.report_callback_sig(name, "event", funcref, args)
 
@@ -1049,18 +1113,41 @@ class Threading:
             q.task_done()
 
     def report_callback_sig(self, name, type, funcref, args):
-
         error_logger = logging.getLogger("Error.{}".format(name))
 
         callback_args = {
-            "scheduler": {"count": 1, "signature": "f(self, kwargs)"},
-            "state": {"count": 5, "signature": "f(self, entity, attribute, old, new, kwargs)"},
-            "event": {"count": 3, "signature": "f(self, event, data, kwargs)"},
-            "log_event": {"count": 6, "signature": "f(self, name, ts, level, type, message, kwargs)"},
-            "initialize": {"count": 0, "signature": "initialize()"},
-            "terminate": {"count": 0, "signature": "terminate()"},
+            "scheduler": {"count": 1, "signature": {True: "f(self, **kwargs)", False: "f(self, kwargs)"}},
+            "state": {
+                "count": 5,
+                "signature": {
+                    True: "f(self, entity, attribute, old, new, **kwargs)",
+                    False: "f(self, entity, attribute, old, new, kwargs)",
+                },
+            },
+            "event": {
+                "count": 3,
+                "signature": {True: "f(self, event, data, **kwargs)", False: "f(self, event, data, kwargs)"},
+            },
+            "log_event": {
+                "count": 6,
+                "signature": {
+                    True: "f(self, name, ts, level, type, message, kwargs)",
+                    False: "f(self, name, ts, level, type, message, kwargs)",
+                },
+            },
+            "initialize": {"count": 0, "signature": {True: "initialize()", False: "initialize()"}},
+            "terminate": {"count": 0, "signature": {True: "terminate()", False: "terminate()"}},
         }
 
+        if "name" in args:
+            app_args = self.AD.app_management.app_config[args["name"]]
+            if "use_dictionary_unpacking" in app_args:
+                use_dictionary_unpacking = app_args["use_dictionary_unpacking"]
+            else:
+                use_dictionary_unpacking = self.AD.use_dictionary_unpacking
+        else:
+            use_dictionary_unpacking = False
+
         try:
             sig = inspect.signature(funcref)
 
@@ -1070,7 +1157,7 @@ class Threading:
                         "Suspect incorrect signature type for callback %s() in %s, should be %s - discarding",
                         funcref.__name__,
                         name,
-                        callback_args[type]["signature"],
+                        callback_args[type]["signature"][use_dictionary_unpacking],
                     )
                 error_logger = logging.getLogger("Error.{}".format(name))
                 error_logger.warning("-" * 60)
